@@ -38,7 +38,6 @@ class Command(object):
         self.__removed = False
         self.__name = name.strip("*")
         self._putters = []
-    
         
     def __deepcopy__(self, memo):
         """
@@ -48,11 +47,12 @@ class Command(object):
         cls = self.__class__
         result = cls.__new__(cls)
         result._unfreeze()
+        memo[id(self)] = result
         result.__dict__ = copy.deepcopy(self.__dict__, memo)
         
         for _ in result._putters:
             _._updateOwner(result)
-
+            
         result._freeze()
         return result
     
@@ -99,19 +99,48 @@ class Command(object):
     
     @property
     def removed(self): return self.__removed
+    
+    def __str__(self): return self.name
 
+class Constant(Command):
+    def __init__(self, name, value):
+        Command.__init__(self, name, False)
+        self.value = value
+        self.usedBy = []
+        self._freeze()
 
+    def _on_kat_remove(self):
+        Command._on_kat_remove(self)
+        self._kat.constants.pop(self.name)
 
-
-
+    def getFinesseText(self):
+        return "const {name} {value}".format(name=self.name, value=self.value)
+    
+    @staticmethod
+    def parseFinesseText(line, kat):
+        v = line.split()
+        
+        if len(v) != 3:
+            raise pkex.BasePyKatException("'{0}' not a valid Finesse const command".format(line))
+        
+        return Constant(v[1], SIfloat(v[2]))
+    
+    @property
+    def value(self): return self.__value
+    @value.setter
+    def value(self, Value): self.__value = SIfloat(Value)
+    
+    def __str__(self): return "$"+self.name
+    
 class variable(Command):
     def __init__(self, name, value):
         Command.__init__(self, name, False)
-        self.__value = value
+        self.value = value
+        
         self._freeze()
         
     def getFinesseText(self):
-        return "variable {name} {value}".format(name=self.name, value=self.value)
+        return "var {name} {value}".format(name=self.name, value=self.value)
     
     @staticmethod
     def parseFinesseText(line, kat):
@@ -152,7 +181,7 @@ class func(Command):
                 rtn.append("noplot " + self.name)
         
             rtn.append("func {name} = {value}".format(name=self.name, value=str(self.value)))
-
+            rtn.extend(self.output._getPutFinesseText())
         return rtn
 
     @staticmethod
@@ -170,12 +199,14 @@ class func(Command):
             
 
 class lock(Command):
-    def __init__(self, name, variable, gain, accuracy, singleLock=False):
+    def __init__(self, name, variable, gain, accuracy, offset=None, singleLock=False):
         Command.__init__(self, name, False)
-        
+
         self.__variable = variable
         self.__gain = gain
         self.__accuracy = accuracy
+        self.__offset = offset
+        
         self.singleLock = singleLock
         self.enabled = True
         self.noplot = False
@@ -185,27 +216,43 @@ class lock(Command):
 
         self._freeze()
         
+    @property
+    def offset(self): return self.__offset
+    @offset.setter
+    def offset(self, value): self.__offset = SIfloat(value)
+    
     @staticmethod
     def parseFinesseText(line, kat):
         v = line.split()
         
-        if len(v) != 5:
+        if len(v) not in (5,6):
             raise pkex.BasePyKatException("'{0}' not a valid Finesse lock command".format(line))
-            
-        return lock(v[1], v[2], SIfloat(v[3]), SIfloat(v[4]), "*" in v[0])
+        
+        if len(v) == 5:
+            return lock(v[1], v[2], SIfloat(v[3]), SIfloat(v[4]), singleLock="*" in v[0])
+        else:
+            return lock(v[1], v[2], SIfloat(v[3]), SIfloat(v[4]), SIfloat(v[5]), singleLock="*" in v[0])
 
 
     def getFinesseText(self):
         if self.enabled:
+            rtn = []
             cmds = "{name} {var} {gain} {accuracy}".format( name=self.name,
                                                             var=str(self.variable),
                                                             gain=str(self.gain),
                                                             accuracy=str(self.accuracy))
             
+            if self.offset is not None:
+                cmds += " %s" % str(self.offset)
+                
             if self.singleLock:
-                return "lock* %s" % cmds
+                rtn.append("lock* %s" % cmds)
             else:
-                return "lock %s" % cmds
+                rtn.append("lock %s" % cmds)
+            
+            rtn.extend(self.output._getPutFinesseText())
+            
+            return rtn
         else:
             return None
 
@@ -238,7 +285,7 @@ class cavity(Command):
         self.enabled = True
 
         self._freeze()
-        
+    
     def getFinesseText(self):
         if self.enabled:
             return 'cav {0} {1} {2} {3} {4}'.format(self.name, self.__c1.name, self.__n1.name, self.__c2.name, self.__n2.name);
@@ -278,8 +325,44 @@ class cavity(Command):
             
         return pykat.commands.cavity(v[1], c1, n1, c2, n2)
         
+    def trace_to_node(self, node):
+        """
+        This method traces a cavity's eigen mode to a
+        given node in the model. It does this by calling
+        the Finesse binary
+    
+        cav_name: cavity name
+        node: node to trace beam to
+        verbose: More or less info
+        base: The kat object to do the tracing with
+        """
+        kat = self._kat.deepcopy()
+        kat.maxtem = 0 # always need a maxtem
+        
+        for cav in kat.getAll(pykat.commands.cavity):
+            if self.name != cav.name:
+                cav.remove()
+                
+        kat.noxaxis = True
+        kat.signals.remove()
+        
+        for _ in kat.detectors:
+            kat.detectors[_].remove()
+            
+        kat.parse("pd p %s" % self.__n1)
+
+        _, T = kat.run(getTraceData=True)
+        qx, qy, _ = T[0][str(node)]
+    
+        return qx, qy
         
 class gauss(object):
+    """
+    Gauss is a differs in behaviour compared to other commands. It does not
+    add itself to the overall kat object, but instead it sets the node
+    components q value directly.
+    """
+    
     @staticmethod
     def parseFinesseText(text, kat):
         
@@ -300,7 +383,7 @@ class gauss(object):
             c = kat.components[component]
             if hasattr(c, node):
                 ns = getattr(c, node)
-                ns.name = name
+                ns.gauss_name = name
             else:
                 raise pkex.BasePyKatException("Component '{0}' is not attached to node {1}".format(component, node))        
         else:
@@ -331,77 +414,7 @@ class gauss(object):
             kat.nodes[node].setGauss(kat.components[component], gp)
         else:
             kat.nodes[node].setGauss(kat.components[component], gpx, gpy)
- 
-# class tf(Command):
-#
-#     class fQ(object):
-#         def __init__(self, f, Q, tf):
-#             assert(tf is not None)
-#             self._tf = tf
-#             self.__f = Param("f", self, None, canFsig=False, isPutable=True, isPutter=False, isTunable=True)
-#             self.__Q = Param("Q", self, None, canFsig=False, isPutable=True, isPutter=False, isTunable=True)
-#
-#         def _register_param(self, param):
-#             self._tf._params.append(param)
-#
-#         @property
-#         def f(self): return self.__f
-#         @f.setter
-#         def f(self,value): self.__f.value = SIfloat(value)
-#
-#         @property
-#         def Q(self): return self.__Q
-#         @Q.setter
-#         def Q(self,value): self.__Q.value = SIfloat(value)
-#
-#     def __init__(self, name):
-#         Command.__init__(self, name, False)
-#         self.zeros = []
-#         self.poles = []
-#         self.gain = 1
-#         self.phase = 0
-#         self._params = []
-#
-#     def addPole(self,f, Q):
-#         self.poles.append(tf.fQ(SIfloat(f), SIfloat(Q), self))
-#
-#     def addZero(self,f, Q):
-#         self.zeros.append(tf.fQ(SIfloat(f), SIfloat(Q), self))
-#
-#     @staticmethod
-#     def parseFinesseText(text):
-#         values = text.split()
-#
-#         if ((len(values)-4) % 3) != 0:
-#             raise pkex.BasePyKatException("Transfer function Finesse code format incorrect '{0}'".format(text))
-#
-#         _tf = tf(values[1])
-#
-#         _tf.gain = SIfloat(values[2])
-#         _tf.phase = SIfloat(values[3])
-#
-#         N = int((len(values)-4) / 3)
-#
-#         for i in range(1,N+1):
-#             if values[i*3+1] == 'p':
-#                 _tf.addPole(SIfloat(values[i*3+2]), SIfloat(values[i*3+3]))
-#             elif values[i*3+1] == 'z':
-#                 _tf.addZero(SIfloat(values[i*3+2]), SIfloat(values[i*3+3]))
-#             else:
-#                 raise pkex.BasePyKatException("Transfer function pole/zero Finesse code format incorrect '{0}'".format(text))
-#
-#         return _tf
-#
-#     def getFinesseText(self):
-#         rtn = "tf {name} {gain} {phase} ".format(name=self.name,gain=self.gain,phase=self.phase)
-#
-#         for p in self.poles:
-#             rtn += "p {f} {Q} ".format(f=p.f, Q=p.Q)
-#
-#         for z in self.zeros:
-#             rtn += "p {f} {Q} ".format(f=z.f, Q=z.Q)
-#
-#         return rtn
+
                    
 class tf(Command):
     
@@ -456,10 +469,94 @@ class tf(Command):
             rtn += "p {f} {Q} ".format(f=p.f, Q=p.Q)
         
         for z in self.zeros:
-            rtn += "p {f} {Q} ".format(f=z.f, Q=z.Q)
+            rtn += "z {f} {Q} ".format(f=z.f, Q=z.Q)
         
         return rtn
         
+class tf2(Command):
+            
+    def __init__(self, name):
+        Command.__init__(self, name, False)
+        self.zeros = []
+        self.poles = []
+        self.gain = 1
+        self.phase = 0
+        self.precision = 4
+        self._freeze()
+    
+    def addPole(self, z):
+        self.poles.append(z)
+    
+    def addZero(self, z):
+        self.zeros.append(z)
+    
+    
+    @staticmethod
+    def parseFinesseText(text):
+        values = text.split()
+        
+        if len(values) != 6:
+            print(values)
+            raise pkex.BasePyKatException("Transfer function Finesse code format incorrect '{0}'".format(text))
+
+        _tf = tf2(values[1])
+        
+        _tf.gain = SIfloat(values[2])
+        _tf.phase = SIfloat(values[3])
+        
+        for _ in values[4].strip("{}").split(','):
+            _ = _.strip()
+            if len(_) > 0:
+                _tf.poles.append(complex(_.replace('i','j')))
+        
+        for _ in values[5].strip("{}").split(','):
+            _ = _.strip()
+            if len(_) > 0:
+                _tf.zeros.append(complex(_.replace('i','j')))
+        
+        return _tf
+        
+    def getFinesseText(self):
+        rtn = "tf2 {name} {gain} {phase} ".format(name=self.name,gain=self.gain,phase=self.phase)
+        
+        fmt = '{0:%i.%if}{1}{2:%i.%if}i'%((self.precision,)*4)
+        c2str = lambda n: fmt.format(n.real, '+' if n.imag>0 else '-', abs(n.imag   ))
+        
+        rtn += "{%s}" % ",".join([c2str(_) for _ in self.poles])
+        rtn += " {%s}" % ",".join([c2str(_) for _ in self.zeros])
+        
+        return rtn
+        
+        
+class tf3(Command):
+            
+    def __init__(self, name, hdf, frequency_dataset, data_dataset):
+        Command.__init__(self, name, False)
+        self.hdf = hdf
+        self.frequency_dataset = frequency_dataset
+        self.data_dataset = data_dataset
+        self._freeze()
+    
+    @staticmethod
+    def parseFinesseText(text):
+        values = text.split()
+        
+        if len(values) != 5:
+            print(values)
+            raise pkex.BasePyKatException("Transfer function Finesse code format incorrect '{0}'".format(text))
+
+        _tf = tf3(*values[1:])
+        
+        return _tf
+        
+    def getFinesseText(self):
+        rtn = "tf3 {name} {hdf} {fds} {dds} ".format(name=self.name,
+                                                     hdf=self.hdf,
+                                                     fds=self.frequency_dataset,
+                                                     dds=self.data_dataset)
+        return rtn
+        
+        
 class xaxis(Command):
     """
     The xaxis object is a unique object to each pykat.finesse.kat instance. It provides
@@ -560,9 +657,14 @@ class xaxis(Command):
         comp_name = self.__comp.name if hasattr(self.__comp, "name") else self.__comp
         param_name = self.__param.name if isinstance(self.__param, Param) else self.__param
         
-        return '{axis_type} {0} {1} {2} {3:.16g} {4:.16g} {5}'.format(
+        rtn =   ['{axis_type} {0} {1} {2} {3:.16g} {4:.16g} {5}'.format(
                 comp_name, param_name, self.scale,
-                self.limits[0], self.limits[1], self.steps, axis_type=self._axis_type);
+                self.limits[0], self.limits[1], self.steps, axis_type=self._axis_type)]
+            
+        rtn.extend(self.x._getPutFinesseText())
+        rtn.extend(self.mx._getPutFinesseText())
+        
+        return rtn
 
 class x2axis(xaxis):
     def __init__(self, scale, limits, param, steps, comp=None, axis_type="x2axis"):
