@@ -102,6 +102,22 @@ class BaseDetector(object) :
             else:
                 raise pkex.BasePyKatException("Nodes should be a list or tuple of node names or a singular node name as a string.")
     
+    # def __deepcopy__(self, memo):
+    #     """
+    #     When deep copying a kat object we need to take into account
+    #     the instance specific properties.
+    #     """
+    #
+    #     # Here we create a copy of this object based of the base class
+    #     # of this one, otherwise we're making a copy of a copy of a copy...
+    #     result = self.__class__.__new__(self.__class__.__base__)
+    #     result._unfreeze()
+    #     memo[id(self)] = result
+    #     result.__dict__ = copy.deepcopy(self.__dict__, memo)
+    #
+    #     result._freeze()
+    #     return result
+    
     def __deepcopy__(self, memo):
         """
         When deep copying a kat object we need to take into account
@@ -111,11 +127,11 @@ class BaseDetector(object) :
         # Here we create a copy of this object based of the base class
         # of this one, otherwise we're making a copy of a copy of a copy...
         result = self.__class__.__new__(self.__class__.__base__)
-        result._unfreeze()
-        memo[id(self)] = result
         result.__dict__ = copy.deepcopy(self.__dict__, memo)
         
-        result._freeze()
+        for _ in result._params:
+            _._updateOwner(result)
+            
         return result
                 
     def _register_param(self, param):
@@ -146,11 +162,16 @@ class BaseDetector(object) :
     def getFinesseText(self):
         """ Base class for individual finesse optical components """    
         #raise NotImplementedError("This function is not implemented")
+        rtn = []
+        
+        for _ in self._mask:
+            if _[0]+_[1] <= self._kat.maxtem:
+                rtn.append("mask {} {} {} {}".format(self.name, _[0], _[1], self._mask[_]))
         
         if self.noplot:
-            return ["noplot " + self.name]
-        else:
-            return []
+            rtn.append("noplot " + self.name)
+            
+        return rtn
         
     def getQGraphicsItem(self):    
         if not USE_GUI:
@@ -174,7 +195,14 @@ class BaseDetector(object) :
     def __str__(self): return self.name
 
     def mask(self, n, m, factor):
-        _id = str(n)+"_"+str(m)
+        """
+        Sets the mode mask to apply to this detector for the mode TEMnm. By default
+        all modes have a factor of 1.
+        
+        n,m - mode indicies
+        factor - float between 0 and 1
+        """
+        _id = (n,m)
         
         # if the mask is 1 then remove this so it doesn't get 
         # printed as by default the value is 1.0
@@ -249,15 +277,19 @@ class beam(Detector1):
         
         self.alternate_beam = alternate_beam
         self.__f = Param("f", self, frequency)        
+        self.__x = Param("x", self, 0)        
+        self.__y = Param("y", self, 0)        
     
         self._freeze()
         
     @property
     def f(self): return self.__f
     
-    @f.setter
-    def f(self, value):
-        self.__f.value = value
+    @property
+    def x(self): return self.__x
+    
+    @property
+    def y(self): return self.__y
         
     @staticmethod
     def parseFinesseText(text):
@@ -574,6 +606,8 @@ class pd(Detector1):
         """
         When deep copying a kat object we need to take into account
         the instance specific properties.
+        
+        PD code needs to override this as it needs to call set_demod_attrs.
         """
         
         # Here we create a copy of this object based of the base class
@@ -581,8 +615,12 @@ class pd(Detector1):
         result = self.__class__.__new__(self.__class__.__base__)
         result.__dict__ = copy.deepcopy(self.__dict__, memo)
         
+        # Need to update f/p attrs
         result.__set_demod_attrs()
         
+        for _ in result._params:
+            _._updateOwner(result)
+            
         return result
 
     def _get_fphi_str(self):
@@ -649,15 +687,15 @@ class pd(Detector1):
         self.__phase4 = Param("phase4", self, None)
         self.__phase5 = Param("phase5", self, None)
         
-        fs = [self.__f1, self.__f2, self.__f3, self.__f4, self.__f5]
-        ps = [self.__phase1, self.__phase2, self.__phase3, self.__phase4, self.__phase5]
+        self.__fs = [self.__f1, self.__f2, self.__f3, self.__f4, self.__f5]
+        self.__ps = [self.__phase1, self.__phase2, self.__phase3, self.__phase4, self.__phase5]
 
         for i in range(num_demods):
             f = 'f{0}'.format(i+1)
             #print("i {0} fs {1} f {2} keys {3}".format(i,len(fs),f, kwargs.keys()))
             
             if f in kwargs:
-                fs[i].value = kwargs[f]
+                self.__fs[i].value = kwargs[f]
             else:
                 raise pkex.BasePyKatException("Missing demodulation frequency {0} (f{0})".format(i+1))    
         
@@ -667,7 +705,7 @@ class pd(Detector1):
                 if kwargs[p] is None and i<num_demods-1:
                     raise pkex.BasePyKatException("Missing demodulation phase {0} (phase{0})".format(i+1))
                     
-                ps[i].value = kwargs[p]
+                self.__ps[i].value = kwargs[p]
             elif i<num_demods-1:
                 raise pkex.BasePyKatException("Missing demodulation phase {0} (phase{0})".format(i+1))
    
