@@ -42,15 +42,21 @@ class MirrorROQWeights:
             
 class surfacemap(object):
     
-    def __init__(self, name, maptype, size=None, center=None, step_size=1.0, scaling=1.0e-9, data=None,
+    def __init__(self, name, maptype, size=None, center=None, step_size=(1,1), scaling=1.0e-9, data=None,
                  notNan=None, zOffset=None, xyOffset=(.0,.0)):
         '''
         size, center, step_size, xyOffset are all tuples of the form (x, y),
         i.e., (col, row).
         '''
+        assert(len(step_size) == 2)
+        assert(step_size[0] > 0 and step_size[1] > 0)
+        assert(len(xyOffset) == 2)
+        assert(size is not None and len(size) == 2)
+        assert(center is not None and len(center) == 2)
         
         self.name = name
         self.type = maptype
+        
         if data is None:
             if size is None:
                 raise BasePyKatException("One of the parameters data or size needs to be specified to create map")
@@ -67,10 +73,11 @@ class surfacemap(object):
         self.notNan = notNan
 
         # Currently "beam center", i.e., mirror_center + xyOffset. 
-        self.center = center
         self.step_size = step_size
+        self.center = center
         self.scaling = scaling
         self._RcRemoved = None
+        
         # Offset of fitted sphere. Proably unnecessary to have here.
         self.zOffset = zOffset
         self.__interp = None
@@ -80,7 +87,7 @@ class surfacemap(object):
 
         self._rom_weights = None
         
-    def write_map(self, filename):
+    def write_map(self, filename, binary=False):
         with open(filename,'w') as mapfile:
             
             mapfile.write("% Surface map\n")
@@ -91,7 +98,7 @@ class surfacemap(object):
             mapfile.write("% Step size (x,y): {0} {1}\n".format(self.step_size[0], self.step_size[1]))
             mapfile.write("% Scaling: {0}\n".format(float(self.scaling)))
             mapfile.write("\n\n")
-            
+
             for i in range(0, self.data.shape[0]):
                 for j in range(0, self.data.shape[1]):
                     mapfile.write("%.15g " % self.data[i,j])
@@ -503,17 +510,17 @@ class surfacemap(object):
         beam center (not mirror center).
         '''
         if w is None:
-            return math.sqrt((self.data[self.notNan]**2).sum())/self.notNan.sum()
+            return math.sqrt((self.data[self.notNan]**2).sum()/self.notNan.sum())
         else:
             R = self.find_radius(unit='meters')
             if w>=R:
-                return math.sqrt((self.data[self.notNan]**2).sum())/self.notNan.sum()
+                return math.sqrt((self.data[self.notNan]**2).sum()/self.notNan.sum())
             else:
                 rho = self.createPolarGrid()[0]
                 inside = np.zeros(self.data.shape,dtype=bool)
                 tmp = rho<w
                 inside[tmp] = self.notNan[tmp]
-                return math.sqrt((self.data[inside]**2).sum())/inside.sum()
+                return math.sqrt((self.data[inside]**2).sum()/inside.sum())
             
     def avg(self, w=None):
         '''
@@ -639,6 +646,8 @@ class surfacemap(object):
         elif isinstance(m,int):
             nVals = range(n,n+1)
             mVals.append(range(m,m+1))
+        else:
+            raise Exception("Unhandled `m` argument: %s" % m)
             
         # Amplitudes
         A = []
@@ -1118,10 +1127,14 @@ class surfacemap(object):
         Based on Simtools function 'FT_create_sphere_for_map.m' by Charlotte Bond.
         '''
         
+        if Rc == np.inf:
+            Rc = 0
+        
         # Adjusting for tilts and offset
         Z = zOffset + (X*np.tan(xTilt) + Y*np.tan(yTilt))/self.scaling
+        
         # Adjusting for spherical shape.
-        if Rc !=0 and Rc is not None:
+        if Rc != 0 and Rc is not None:
             Z = Z + (Rc - np.sign(Rc)*np.sqrt(Rc**2 - (X-x0)**2 - (Y-y0)**2))/self.scaling
         
         if isPlot:
@@ -1351,9 +1364,9 @@ class surfacemap(object):
             mapfile.write('Date:  {:s}\n'.format(time.strftime("%d/%m/%Y  %H:%M:%S")))
             mapfile.write('---------------------------------------\n')
             if w is None:
-                mapfile.write('Weights:   None')
+                mapfile.write('Weights:   None\n')
             else:
-                mapfile.write('Weights:   r     = {:.2f} cm'.format(w*100))
+                mapfile.write('Weights:   r     = {:.2f} cm\n'.format(w*100))
             mapfile.write('Diameter:  D     = {:.2f} cm\n'.format(self.find_radius(unit='meters')*200.0))
             mapfile.write('Offset:    A00   = {:.2f} nm\n'.format(self.zernikeRemoved['00'][2]))
             mapfile.write('Tilt:      A1-1  = {:.2f} nm\n'.format(self.zernikeRemoved['-11'][2]))
@@ -1399,9 +1412,9 @@ class surfacemap(object):
         Returns: A
         A  - List of amplitudes of the removed Zernike polynomials.
         '''
-        if m=='all':
-            m = range(-n,n+1,2)
-            
+        if m == 'all':
+            m = list(range(-n,n+1,2))
+
         R = self.find_radius(unit='meters')
         A = self.zernikeConvol(n,m)
         rho, phi= self.createPolarGrid()
@@ -1412,15 +1425,11 @@ class surfacemap(object):
                 Z = zernike(m[k], n, rho, phi)
                 self.data[self.notNan] = self.data[self.notNan]-A[k]*Z[self.notNan]
                 self.zernikeRemoved = (m[k], n, A[k])
-#        elif isinstance(m, range):
-#            for k in range(len(m)):
-#                Z = zernike(m[k], n, rho, phi)
-#                self.data[self.notNan] = self.data[self.notNan]-A[k]*Z[self.notNan]
-#                self.zernikeRemoved = (m[k], n, A[k])
         else:
             Z = zernike(m, n, rho, phi)
             self.data[self.notNan] = self.data[self.notNan]-A*Z[self.notNan]
             self.zernikeRemoved = (m, n, A)
+            
         return A
         
 class mergedmap:
@@ -1683,19 +1692,29 @@ class aperturemap(surfacemap):
         
         
 class curvedmap(surfacemap):
+    """
+    Example:
+        import pykat
+        from pykat.optics.knm import *
+        from pykat.optics.maps import *
+
+        N = 501
+        dx = 1/float(N)
+        Roc = 10e3
     
+        m = curvedmap("test", (N,N), (dx,dx), RoC)
+    """
     def __init__(self, name, size, step_size, Rc):
         surfacemap.__init__(self, name, "phase reflection", size, (np.array(size)+1)/2.0, step_size, 1e-6)
         
         self.Rc = Rc
-        
+            
     @property
     def Rc(self):
         return self.__Rc
     
     @Rc.setter
     def Rc(self, value):
-        
         self.__Rc = float(value)
     
         xx, yy = np.meshgrid(self.x, self.y)
